@@ -16,11 +16,12 @@
 # along with Paramiko; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 
+# flake8: noqa
 import sys
 from paramiko._version import __version__, __version_info__
 
 if sys.version_info < (2, 6):
-    raise RuntimeError('You need Python 2.6+ for this module.')
+    raise RuntimeError("You need Python 2.6+ for this module.")
 
 
 __author__ = "Jeff Forcier <jeff@bitprophet.org>"
@@ -28,16 +29,30 @@ __license__ = "GNU Lesser General Public License (LGPL)"
 
 
 from paramiko.transport import SecurityOptions, Transport
-from paramiko.client import SSHClient, MissingHostKeyPolicy, AutoAddPolicy, RejectPolicy, WarningPolicy
+from paramiko.client import (
+    SSHClient,
+    MissingHostKeyPolicy,
+    AutoAddPolicy,
+    RejectPolicy,
+    WarningPolicy,
+)
 from paramiko.auth_handler import AuthHandler
+from paramiko.ssh_gss import GSSAuth, GSS_AUTH_AVAILABLE, GSS_EXCEPTIONS
 from paramiko.channel import Channel, ChannelFile
-from paramiko.ssh_exception import SSHException, PasswordRequiredException, \
-    BadAuthenticationType, ChannelException, BadHostKeyException, \
-    AuthenticationException, ProxyCommandFailure
+from paramiko.ssh_exception import (
+    SSHException,
+    PasswordRequiredException,
+    BadAuthenticationType,
+    ChannelException,
+    BadHostKeyException,
+    AuthenticationException,
+    ProxyCommandFailure,
+)
 from paramiko.server import ServerInterface, SubsystemHandler, InteractiveQuery
 from paramiko.rsakey import RSAKey
 from paramiko.dsskey import DSSKey
 from paramiko.ecdsakey import ECDSAKey
+from paramiko.ed25519key import Ed25519Key
 from paramiko.sftp import SFTPError, BaseSFTP
 from paramiko.sftp_client import SFTP, SFTPClient
 from paramiko.sftp_server import SFTPServer
@@ -49,54 +64,72 @@ from paramiko.message import Message
 from paramiko.packet import Packetizer
 from paramiko.file import BufferedFile
 from paramiko.agent import Agent, AgentKey
-from paramiko.pkey import PKey
+from paramiko.pkey import PKey, PublicBlob
 from paramiko.hostkeys import HostKeys
 from paramiko.config import SSHConfig
 from paramiko.proxy import ProxyCommand
 
-from paramiko.common import AUTH_SUCCESSFUL, AUTH_PARTIALLY_SUCCESSFUL, AUTH_FAILED, \
-    OPEN_SUCCEEDED, OPEN_FAILED_ADMINISTRATIVELY_PROHIBITED, OPEN_FAILED_CONNECT_FAILED, \
-    OPEN_FAILED_UNKNOWN_CHANNEL_TYPE, OPEN_FAILED_RESOURCE_SHORTAGE
+from paramiko.common import (
+    AUTH_SUCCESSFUL,
+    AUTH_PARTIALLY_SUCCESSFUL,
+    AUTH_FAILED,
+    OPEN_SUCCEEDED,
+    OPEN_FAILED_ADMINISTRATIVELY_PROHIBITED,
+    OPEN_FAILED_CONNECT_FAILED,
+    OPEN_FAILED_UNKNOWN_CHANNEL_TYPE,
+    OPEN_FAILED_RESOURCE_SHORTAGE,
+)
 
-from paramiko.sftp import SFTP_OK, SFTP_EOF, SFTP_NO_SUCH_FILE, SFTP_PERMISSION_DENIED, SFTP_FAILURE, \
-    SFTP_BAD_MESSAGE, SFTP_NO_CONNECTION, SFTP_CONNECTION_LOST, SFTP_OP_UNSUPPORTED
+from paramiko.sftp import (
+    SFTP_OK,
+    SFTP_EOF,
+    SFTP_NO_SUCH_FILE,
+    SFTP_PERMISSION_DENIED,
+    SFTP_FAILURE,
+    SFTP_BAD_MESSAGE,
+    SFTP_NO_CONNECTION,
+    SFTP_CONNECTION_LOST,
+    SFTP_OP_UNSUPPORTED,
+)
 
 from paramiko.common import io_sleep
 
-__all__ = [ 'Transport',
-            'SSHClient',
-            'MissingHostKeyPolicy',
-            'AutoAddPolicy',
-            'RejectPolicy',
-            'WarningPolicy',
-            'SecurityOptions',
-            'SubsystemHandler',
-            'Channel',
-            'PKey',
-            'RSAKey',
-            'DSSKey',
-            'Message',
-            'SSHException',
-            'AuthenticationException',
-            'PasswordRequiredException',
-            'BadAuthenticationType',
-            'ChannelException',
-            'BadHostKeyException',
-            'ProxyCommand',
-            'ProxyCommandFailure',
-            'SFTP',
-            'SFTPFile',
-            'SFTPHandle',
-            'SFTPClient',
-            'SFTPServer',
-            'SFTPError',
-            'SFTPAttributes',
-            'SFTPServerInterface',
-            'ServerInterface',
-            'BufferedFile',
-            'Agent',
-            'AgentKey',
-            'HostKeys',
-            'SSHConfig',
-            'util',
-            'io_sleep' ]
+__all__ = [
+    "Transport",
+    "SSHClient",
+    "MissingHostKeyPolicy",
+    "AutoAddPolicy",
+    "RejectPolicy",
+    "WarningPolicy",
+    "SecurityOptions",
+    "SubsystemHandler",
+    "Channel",
+    "PKey",
+    "RSAKey",
+    "DSSKey",
+    "Message",
+    "SSHException",
+    "AuthenticationException",
+    "PasswordRequiredException",
+    "BadAuthenticationType",
+    "ChannelException",
+    "BadHostKeyException",
+    "ProxyCommand",
+    "ProxyCommandFailure",
+    "SFTP",
+    "SFTPFile",
+    "SFTPHandle",
+    "SFTPClient",
+    "SFTPServer",
+    "SFTPError",
+    "SFTPAttributes",
+    "SFTPServerInterface",
+    "ServerInterface",
+    "BufferedFile",
+    "Agent",
+    "AgentKey",
+    "HostKeys",
+    "SSHConfig",
+    "util",
+    "io_sleep",
+]
