@@ -25,7 +25,8 @@ from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric import ec
 from cryptography.hazmat.primitives.asymmetric.utils import (
-    decode_dss_signature, encode_dss_signature
+    decode_dss_signature,
+    encode_dss_signature,
 )
 
 from paramiko.common import four_byte
@@ -43,6 +44,7 @@ class _ECDSACurve(object):
     the proper hash function. Also grabs the proper curve from the 'ecdsa'
     package.
     """
+
     def __init__(self, curve_class, nist_name):
         self.nist_name = nist_name
         self.key_length = curve_class.key_size
@@ -67,6 +69,7 @@ class _ECDSACurveSet(object):
     format identifier. The two ways in which ECDSAKey needs to be able to look
     up curves.
     """
+
     def __init__(self, ecdsa_curves):
         self.ecdsa_curves = ecdsa_curves
 
@@ -95,16 +98,27 @@ class ECDSAKey(PKey):
     data.
     """
 
-    _ECDSA_CURVES = _ECDSACurveSet([
-        _ECDSACurve(ec.SECP256R1, 'nistp256'),
-        _ECDSACurve(ec.SECP384R1, 'nistp384'),
-        _ECDSACurve(ec.SECP521R1, 'nistp521'),
-    ])
+    _ECDSA_CURVES = _ECDSACurveSet(
+        [
+            _ECDSACurve(ec.SECP256R1, "nistp256"),
+            _ECDSACurve(ec.SECP384R1, "nistp384"),
+            _ECDSACurve(ec.SECP521R1, "nistp521"),
+        ]
+    )
 
-    def __init__(self, msg=None, data=None, filename=None, password=None,
-                 vals=None, file_obj=None, validate_point=True):
+    def __init__(
+        self,
+        msg=None,
+        data=None,
+        filename=None,
+        password=None,
+        vals=None,
+        file_obj=None,
+        validate_point=True,
+    ):
         self.verifying_key = None
         self.signing_key = None
+        self.public_blob = None
         if file_obj is not None:
             self._from_private_key(file_obj, password)
             return
@@ -118,12 +132,26 @@ class ECDSAKey(PKey):
             c_class = self.signing_key.curve.__class__
             self.ecdsa_curve = self._ECDSA_CURVES.get_by_curve_class(c_class)
         else:
-            if msg is None:
-                raise SSHException('Key object may not be empty')
+            # Must set ecdsa_curve first; subroutines called herein may need to
+            # spit out our get_name(), which relies on this.
+            key_type = msg.get_text()
+            # But this also means we need to hand it a real key/curve
+            # identifier, so strip out any cert business. (NOTE: could push
+            # that into _ECDSACurveSet.get_by_key_format_identifier(), but it
+            # feels more correct to do it here?)
+            suffix = "-cert-v01@openssh.com"
+            if key_type.endswith(suffix):
+                key_type = key_type[: -len(suffix)]
             self.ecdsa_curve = self._ECDSA_CURVES.get_by_key_format_identifier(
-                msg.get_text())
-            if self.ecdsa_curve is None:
-                raise SSHException('Invalid key')
+                key_type
+            )
+            key_types = self._ECDSA_CURVES.get_key_format_identifier_list()
+            cert_types = [
+                "{0}-cert-v01@openssh.com".format(x) for x in key_types
+            ]
+            self._check_type_and_load_cert(
+                msg=msg, key_type=key_types, cert_type=cert_types
+            )
             curvename = msg.get_text()
             if curvename != self.ecdsa_curve.nist_name:
                 raise SSHException("Can't handle curve of type %s" % curvename)
@@ -152,10 +180,10 @@ class ECDSAKey(PKey):
         key_size_bytes = (key.curve.key_size + 7) // 8
 
         x_bytes = deflate_long(numbers.x, add_sign_padding=False)
-        x_bytes = b'\x00' * (key_size_bytes - len(x_bytes)) + x_bytes
+        x_bytes = b"\x00" * (key_size_bytes - len(x_bytes)) + x_bytes
 
         y_bytes = deflate_long(numbers.y, add_sign_padding=False)
-        y_bytes = b'\x00' * (key_size_bytes - len(y_bytes)) + y_bytes
+        y_bytes = b"\x00" * (key_size_bytes - len(y_bytes)) + y_bytes
 
         point_str = four_byte + x_bytes + y_bytes
         m.add_string(point_str)
@@ -165,10 +193,13 @@ class ECDSAKey(PKey):
         return self.asbytes()
 
     def __hash__(self):
-        h = hash(self.get_name())
-        h = h * 37 + hash(self.verifying_key.public_numbers().x)
-        h = h * 37 + hash(self.verifying_key.public_numbers().y)
-        return hash(h)
+        return hash(
+            (
+                self.get_name(),
+                self.verifying_key.public_numbers().x,
+                self.verifying_key.public_numbers().y,
+            )
+        )
 
     def get_name(self):
         return self.ecdsa_curve.key_format_identifier
@@ -181,9 +212,7 @@ class ECDSAKey(PKey):
 
     def sign_ssh_data(self, data):
         ecdsa = ec.ECDSA(self.ecdsa_curve.hash_object())
-        signer = self.signing_key.signer(ecdsa)
-        signer.update(data)
-        sig = signer.finalize()
+        sig = self.signing_key.sign(data, ecdsa)
         r, s = decode_dss_signature(sig)
 
         m = Message()
@@ -198,12 +227,10 @@ class ECDSAKey(PKey):
         sigR, sigS = self._sigdecode(sig)
         signature = encode_dss_signature(sigR, sigS)
 
-        verifier = self.verifying_key.verifier(
-            signature, ec.ECDSA(self.ecdsa_curve.hash_object())
-        )
-        verifier.update(data)
         try:
-            verifier.verify()
+            self.verifying_key.verify(
+                signature, data, ec.ECDSA(self.ecdsa_curve.hash_object())
+            )
         except InvalidSignature:
             return False
         else:
@@ -214,7 +241,7 @@ class ECDSAKey(PKey):
             filename,
             self.signing_key,
             serialization.PrivateFormat.TraditionalOpenSSL,
-            password=password
+            password=password,
         )
 
     def write_private_key(self, file_obj, password=None):
@@ -222,7 +249,7 @@ class ECDSAKey(PKey):
             file_obj,
             self.signing_key,
             serialization.PrivateFormat.TraditionalOpenSSL,
-            password=password
+            password=password,
         )
 
     @classmethod
@@ -231,26 +258,26 @@ class ECDSAKey(PKey):
         Generate a new private ECDSA key.  This factory function can be used to
         generate a new host key or authentication key.
 
-        :param function progress_func: Not used for this type of key.
+        :param progress_func: Not used for this type of key.
         :returns: A new private key (`.ECDSAKey`) object
         """
         if bits is not None:
             curve = cls._ECDSA_CURVES.get_by_key_length(bits)
             if curve is None:
-                raise ValueError("Unsupported key length: %d"%(bits))
+                raise ValueError("Unsupported key length: %d" % bits)
             curve = curve.curve_class()
 
         private_key = ec.generate_private_key(curve, backend=default_backend())
         return ECDSAKey(vals=(private_key, private_key.public_key()))
 
-    ###  internals...
+    # ...internals...
 
     def _from_private_key_file(self, filename, password):
-        data = self._read_private_key_file('EC', filename, password)
+        data = self._read_private_key_file("EC", filename, password)
         self._decode_key(data)
 
     def _from_private_key(self, file_obj, password):
-        data = self._read_private_key('EC', file_obj, password)
+        data = self._read_private_key("EC", file_obj, password)
         self._decode_key(data)
 
     def _decode_key(self, data):
@@ -258,7 +285,7 @@ class ECDSAKey(PKey):
             key = serialization.load_der_private_key(
                 data, password=None, backend=default_backend()
             )
-        except ValueError as e:
+        except (ValueError, AssertionError) as e:
             raise SSHException(str(e))
 
         self.signing_key = key
