@@ -15,14 +15,19 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with Paramiko; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
-from paramiko.common import linefeed_byte_value, crlf, cr_byte, linefeed_byte, \
-    cr_byte_value
-from paramiko.py3compat import BytesIO, PY2, u, b, bytes_types
+from paramiko.common import (
+    linefeed_byte_value,
+    crlf,
+    cr_byte,
+    linefeed_byte,
+    cr_byte_value,
+)
+from paramiko.py3compat import BytesIO, PY2, u, bytes_types, text_type
 
 from paramiko.util import ClosingContextManager
 
 
-class BufferedFile (ClosingContextManager):
+class BufferedFile(ClosingContextManager):
     """
     Reusable base class to implement Python-style file buffering around a
     simpler stream.
@@ -59,17 +64,17 @@ class BufferedFile (ClosingContextManager):
 
     def __del__(self):
         self.close()
-        
+
     def __iter__(self):
         """
         Returns an iterator that can be used to iterate over the lines in this
         file.  This iterator happens to return the file itself, since a file is
         its own iterator.
 
-        :raises ValueError: if the file is closed.
+        :raises: ``ValueError`` -- if the file is closed.
         """
         if self._closed:
-            raise ValueError('I/O operation on closed file')
+            raise ValueError("I/O operation on closed file")
         return self
 
     def close(self):
@@ -89,28 +94,14 @@ class BufferedFile (ClosingContextManager):
         return
 
     if PY2:
+
         def next(self):
             """
             Returns the next line from the input, or raises
-            `~exceptions.StopIteration` when EOF is hit.  Unlike Python file
+            ``StopIteration`` when EOF is hit.  Unlike Python file
             objects, it's okay to mix calls to `next` and `readline`.
 
-            :raises StopIteration: when the end of the file is reached.
-
-            :return: a line (`str`) read from the file.
-            """
-            line = self.readline()
-            if not line:
-                raise StopIteration
-            return line
-    else:
-        def __next__(self):
-            """
-            Returns the next line from the input, or raises `.StopIteration` when
-            EOF is hit.  Unlike python file objects, it's okay to mix calls to
-            `.next` and `.readline`.
-
-            :raises StopIteration: when the end of the file is reached.
+            :raises: ``StopIteration`` -- when the end of the file is reached.
 
             :returns: a line (`str`) read from the file.
             """
@@ -119,11 +110,70 @@ class BufferedFile (ClosingContextManager):
                 raise StopIteration
             return line
 
+    else:
+
+        def __next__(self):
+            """
+            Returns the next line from the input, or raises ``StopIteration``
+            when EOF is hit.  Unlike python file objects, it's okay to mix
+            calls to `.next` and `.readline`.
+
+            :raises: ``StopIteration`` -- when the end of the file is reached.
+
+            :returns: a line (`str`) read from the file.
+            """
+            line = self.readline()
+            if not line:
+                raise StopIteration
+            return line
+
+    def readable(self):
+        """
+        Check if the file can be read from.
+
+        :returns:
+            `True` if the file can be read from. If `False`, `read` will raise
+            an exception.
+        """
+        return (self._flags & self.FLAG_READ) == self.FLAG_READ
+
+    def writable(self):
+        """
+        Check if the file can be written to.
+
+        :returns:
+            `True` if the file can be written to. If `False`, `write` will
+            raise an exception.
+        """
+        return (self._flags & self.FLAG_WRITE) == self.FLAG_WRITE
+
+    def seekable(self):
+        """
+        Check if the file supports random access.
+
+        :returns:
+            `True` if the file supports random access. If `False`, `seek` will
+            raise an exception.
+        """
+        return False
+
+    def readinto(self, buff):
+        """
+        Read up to ``len(buff)`` bytes into ``bytearray`` *buff* and return the
+        number of bytes read.
+
+        :returns:
+            The number of bytes read.
+        """
+        data = self.read(len(buff))
+        buff[: len(data)] = data
+        return len(data)
+
     def read(self, size=None):
         """
-        Read at most ``size`` bytes from the file (less if we hit the end of the
-        file first).  If the ``size`` argument is negative or omitted, read all
-        the remaining data in the file.
+        Read at most ``size`` bytes from the file (less if we hit the end of
+        the file first).  If the ``size`` argument is negative or omitted,
+        read all the remaining data in the file.
 
         .. note::
             ``'b'`` mode flag is ignored (``self.FLAG_BINARY`` in
@@ -132,14 +182,14 @@ class BufferedFile (ClosingContextManager):
             text data.
 
         :param int size: maximum number of bytes to read
-        :return:
+        :returns:
             data read from the file (as bytes), or an empty string if EOF was
             encountered immediately
         """
         if self._closed:
-            raise IOError('File is closed')
+            raise IOError("File is closed")
         if not (self._flags & self.FLAG_READ):
-            raise IOError('File is not open for reading')
+            raise IOError("File is not open for reading")
         if (size is None) or (size < 0):
             # go for broke
             result = self._rbuffer
@@ -155,12 +205,12 @@ class BufferedFile (ClosingContextManager):
                 result += new_data
                 self._realpos += len(new_data)
                 self._pos += len(new_data)
-            return result 
+            return result
         if size <= len(self._rbuffer):
             result = self._rbuffer[:size]
             self._rbuffer = self._rbuffer[size:]
             self._pos += len(result)
-            return result 
+            return result
         while len(self._rbuffer) < size:
             read_size = size - len(self._rbuffer)
             if self._flags & self.FLAG_BUFFERED:
@@ -176,7 +226,7 @@ class BufferedFile (ClosingContextManager):
         result = self._rbuffer[:size]
         self._rbuffer = self._rbuffer[size:]
         self._pos += len(result)
-        return result 
+        return result
 
     def readline(self, size=None):
         """
@@ -192,7 +242,7 @@ class BufferedFile (ClosingContextManager):
             characters (``'\\0'``) if they occurred in the input.
 
         :param int size: maximum length of returned string.
-        :return:
+        :returns:
             next line of the file, or an empty string if the end of the
             file has been reached.
 
@@ -202,12 +252,17 @@ class BufferedFile (ClosingContextManager):
         """
         # it's almost silly how complex this function is.
         if self._closed:
-            raise IOError('File is closed')
+            raise IOError("File is closed")
         if not (self._flags & self.FLAG_READ):
-            raise IOError('File not open for reading')
+            raise IOError("File not open for reading")
         line = self._rbuffer
+        truncated = False
         while True:
-            if self._at_trailing_cr and (self._flags & self.FLAG_UNIVERSAL_NEWLINE) and (len(line) > 0):
+            if (
+                self._at_trailing_cr
+                and self._flags & self.FLAG_UNIVERSAL_NEWLINE
+                and len(line) > 0
+            ):
                 # edge case: the newline may be '\r\n' and we may have read
                 # only the first '\r' last time.
                 if line[0] == linefeed_byte_value:
@@ -220,15 +275,17 @@ class BufferedFile (ClosingContextManager):
             # enough.
             if (size is not None) and (size >= 0):
                 if len(line) >= size:
-                    # truncate line and return
+                    # truncate line
                     self._rbuffer = line[size:]
                     line = line[:size]
-                    self._pos += len(line)
-                    return line if self._flags & self.FLAG_BINARY else u(line)
+                    truncated = True
+                    break
                 n = size - len(line)
             else:
                 n = self._bufsize
-            if (linefeed_byte in line) or ((self._flags & self.FLAG_UNIVERSAL_NEWLINE) and (cr_byte in line)):
+            if linefeed_byte in line or (
+                self._flags & self.FLAG_UNIVERSAL_NEWLINE and cr_byte in line
+            ):
                 break
             try:
                 new_data = self._read(n)
@@ -246,10 +303,25 @@ class BufferedFile (ClosingContextManager):
             rpos = line.find(cr_byte)
             if (rpos >= 0) and (rpos < pos or pos < 0):
                 pos = rpos
+        if pos == -1:
+            # we couldn't find a newline in the truncated string, return it
+            self._pos += len(line)
+            return line if self._flags & self.FLAG_BINARY else u(line)
         xpos = pos + 1
-        if (line[pos] == cr_byte_value) and (xpos < len(line)) and (line[xpos] == linefeed_byte_value):
+        if (
+            line[pos] == cr_byte_value
+            and xpos < len(line)
+            and line[xpos] == linefeed_byte_value
+        ):
             xpos += 1
-        self._rbuffer = line[xpos:]
+        # if the string was truncated, _rbuffer needs to have the string after
+        # the newline character plus the truncated part of the line we stored
+        # earlier in _rbuffer
+        if truncated:
+            self._rbuffer = line[xpos:] + self._rbuffer
+        else:
+            self._rbuffer = line[xpos:]
+
         lf = line[pos:xpos]
         line = line[:pos] + linefeed_byte
         if (len(self._rbuffer) == 0) and (lf == cr_byte):
@@ -269,7 +341,7 @@ class BufferedFile (ClosingContextManager):
         after rounding up to an internal buffer size) are read.
 
         :param int sizehint: desired maximum number of bytes to read.
-        :return: `list` of lines read from the file.
+        :returns: `list` of lines read from the file.
         """
         lines = []
         byte_count = 0
@@ -292,16 +364,16 @@ class BufferedFile (ClosingContextManager):
             If a file is opened in append mode (``'a'`` or ``'a+'``), any seek
             operations will be undone at the next write (as the file position
             will move back to the end of the file).
-        
+
         :param int offset:
             position to move to within the file, relative to ``whence``.
         :param int whence:
             type of movement: 0 = absolute; 1 = relative to the current
             position; 2 = relative to the end of the file.
 
-        :raises IOError: if the file doesn't support random access.
+        :raises: ``IOError`` -- if the file doesn't support random access.
         """
-        raise IOError('File does not support seeking.')
+        raise IOError("File does not support seeking.")
 
     def tell(self):
         """
@@ -309,7 +381,7 @@ class BufferedFile (ClosingContextManager):
         useful if the underlying file doesn't support random access, or was
         opened in append mode.
 
-        :return: file position (`number <int>` of bytes).
+        :returns: file position (`number <int>` of bytes).
         """
         return self._pos
 
@@ -320,13 +392,15 @@ class BufferedFile (ClosingContextManager):
         written yet.  (Use `flush` or `close` to force buffered data to be
         written out.)
 
-        :param str data: data to write
+        :param data: ``str``/``bytes`` data to write
         """
-        data = b(data)
+        if isinstance(data, text_type):
+            # Accept text and encode as utf-8 for compatibility only.
+            data = data.encode("utf-8")
         if self._closed:
-            raise IOError('File is closed')
+            raise IOError("File is closed")
         if not (self._flags & self.FLAG_WRITE):
-            raise IOError('File not open for writing')
+            raise IOError("File not open for writing")
         if not (self._flags & self.FLAG_BUFFERED):
             self._write_all(data)
             return
@@ -337,9 +411,9 @@ class BufferedFile (ClosingContextManager):
             if last_newline_pos >= 0:
                 wbuf = self._wbuffer.getvalue()
                 last_newline_pos += len(wbuf) - len(data)
-                self._write_all(wbuf[:last_newline_pos + 1])
+                self._write_all(wbuf[: last_newline_pos + 1])
                 self._wbuffer = BytesIO()
-                self._wbuffer.write(wbuf[last_newline_pos + 1:])
+                self._wbuffer.write(wbuf[last_newline_pos + 1 :])
             return
         # even if we're line buffering, if the buffer has grown past the
         # buffer size, force a flush.
@@ -354,7 +428,7 @@ class BufferedFile (ClosingContextManager):
         name is intended to match `readlines`; `writelines` does not add line
         separators.)
 
-        :param iterable sequence: an iterable sequence of strings.
+        :param sequence: an iterable sequence of strings.
         """
         for line in sequence:
             self.write(line)
@@ -371,7 +445,7 @@ class BufferedFile (ClosingContextManager):
     def closed(self):
         return self._closed
 
-    ###  overrides...
+    # ...overrides...
 
     def _read(self, size):
         """
@@ -386,7 +460,7 @@ class BufferedFile (ClosingContextManager):
         (subclass override)
         Write data into the stream.
         """
-        raise IOError('write not implemented')
+        raise IOError("write not implemented")
 
     def _get_size(self):
         """
@@ -399,9 +473,9 @@ class BufferedFile (ClosingContextManager):
         """
         return 0
 
-    ###  internals...
+    # ...internals...
 
-    def _set_mode(self, mode='r', bufsize=-1):
+    def _set_mode(self, mode="r", bufsize=-1):
         """
         Subclasses call this method to initialize the BufferedFile.
         """
@@ -424,17 +498,17 @@ class BufferedFile (ClosingContextManager):
             # unbuffered
             self._flags &= ~(self.FLAG_BUFFERED | self.FLAG_LINE_BUFFERED)
 
-        if ('r' in mode) or ('+' in mode):
+        if ("r" in mode) or ("+" in mode):
             self._flags |= self.FLAG_READ
-        if ('w' in mode) or ('+' in mode):
+        if ("w" in mode) or ("+" in mode):
             self._flags |= self.FLAG_WRITE
-        if 'a' in mode:
+        if "a" in mode:
             self._flags |= self.FLAG_WRITE | self.FLAG_APPEND
             self._size = self._get_size()
             self._pos = self._realpos = self._size
-        if 'b' in mode:
+        if "b" in mode:
             self._flags |= self.FLAG_BINARY
-        if 'U' in mode:
+        if "U" in mode:
             self._flags |= self.FLAG_UNIVERSAL_NEWLINE
             # built-in file objects have this attribute to store which kinds of
             # line terminations they've seen:
@@ -463,7 +537,9 @@ class BufferedFile (ClosingContextManager):
             return
         if self.newlines is None:
             self.newlines = newline
-        elif self.newlines != newline and isinstance(self.newlines, bytes_types):
+        elif self.newlines != newline and isinstance(
+            self.newlines, bytes_types
+        ):
             self.newlines = (self.newlines, newline)
         elif newline not in self.newlines:
             self.newlines += (newline,)
