@@ -30,10 +30,11 @@ from paramiko.dsskey import DSSKey
 from paramiko.rsakey import RSAKey
 from paramiko.util import get_logger, constant_time_bytes_eq
 from paramiko.ecdsakey import ECDSAKey
+from paramiko.ed25519key import Ed25519Key
 from paramiko.ssh_exception import SSHException
 
 
-class HostKeys (MutableMapping):
+class HostKeys(MutableMapping):
     """
     Representation of an OpenSSH-style "known hosts" file.  Host keys can be
     read from one or more files, and then individual hosts can be looked up to
@@ -85,12 +86,12 @@ class HostKeys (MutableMapping):
 
         :param str filename: name of the file to read host keys from
 
-        :raises IOError: if there was an error reading the file
+        :raises: ``IOError`` -- if there was an error reading the file
         """
-        with open(filename, 'r') as f:
+        with open(filename, "r") as f:
             for lineno, line in enumerate(f, 1):
                 line = line.strip()
-                if (len(line) == 0) or (line[0] == '#'):
+                if (len(line) == 0) or (line[0] == "#"):
                     continue
                 try:
                     e = HostKeyEntry.from_line(line, lineno)
@@ -106,18 +107,18 @@ class HostKeys (MutableMapping):
 
     def save(self, filename):
         """
-        Save host keys into a file, in the format used by OpenSSH.  The order of
-        keys in the file will be preserved when possible (if these keys were
+        Save host keys into a file, in the format used by OpenSSH.  The order
+        of keys in the file will be preserved when possible (if these keys were
         loaded from a file originally).  The single exception is that combined
         lines will be split into individual key lines, which is arguably a bug.
 
         :param str filename: name of the file to write
 
-        :raises IOError: if there was an error writing the file
+        :raises: ``IOError`` -- if there was an error writing the file
 
         .. versionadded:: 1.6.1
         """
-        with open(filename, 'w') as f:
+        with open(filename, "w") as f:
             for e in self._entries:
                 line = e.to_line()
                 if line:
@@ -130,9 +131,11 @@ class HostKeys (MutableMapping):
         returned.  The keytype will be either ``"ssh-rsa"`` or ``"ssh-dss"``.
 
         :param str hostname: the hostname (or IP) to lookup
-        :return: dict of `str` -> `.PKey` keys associated with this host (or ``None``)
+        :return: dict of `str` -> `.PKey` keys associated with this host
+            (or ``None``)
         """
-        class SubDict (MutableMapping):
+
+        class SubDict(MutableMapping):
             def __init__(self, hostname, entries, hostkeys):
                 self._hostname = hostname
                 self._entries = entries
@@ -173,17 +176,36 @@ class HostKeys (MutableMapping):
                     self._hostkeys._entries.append(e)
 
             def keys(self):
-                return [e.key.get_name() for e in self._entries if e.key is not None]
+                return [
+                    e.key.get_name()
+                    for e in self._entries
+                    if e.key is not None
+                ]
 
         entries = []
         for e in self._entries:
-            for h in e.hostnames:
-                if h.startswith('|1|') and not hostname.startswith('|1|') and constant_time_bytes_eq(self.hash_host(hostname, h), h) or h == hostname:
-                    entries.append(e)
+            if self._hostname_matches(hostname, e):
+                entries.append(e)
         if len(entries) == 0:
             return None
         return SubDict(hostname, entries, self)
 
+    def _hostname_matches(self, hostname, entry):
+        """
+        Tests whether ``hostname`` string matches given SubDict ``entry``.
+
+        :returns bool:
+        """
+        for h in entry.hostnames:
+            if (
+                h == hostname
+                or h.startswith("|1|")
+                and not hostname.startswith("|1|")
+                and constant_time_bytes_eq(self.hash_host(hostname, h), h)
+            ):
+                return True
+        return False
+
     def check(self, hostname, key):
         """
         Return True if the given key is associated with the given hostname
@@ -215,15 +237,22 @@ class HostKeys (MutableMapping):
     def __len__(self):
         return len(self.keys())
 
-    def __delitem__(self, key):
-        k = self[key]
-
     def __getitem__(self, key):
         ret = self.lookup(key)
         if ret is None:
             raise KeyError(key)
         return ret
 
+    def __delitem__(self, key):
+        index = None
+        for i, entry in enumerate(self._entries):
+            if self._hostname_matches(key, entry):
+                index = i
+                break
+        if index is None:
+            raise KeyError(key)
+        self._entries.pop(index)
+
     def __setitem__(self, hostname, entry):
         # don't use this please.
         if len(entry) == 0:
@@ -232,7 +261,7 @@ class HostKeys (MutableMapping):
         for key_type in entry.keys():
             found = False
             for e in self._entries:
-                if (hostname in e.hostnames) and (e.key.get_name() == key_type):
+                if (hostname in e.hostnames) and e.key.get_name() == key_type:
                     # replace
                     e.key = entry[key_type]
                     found = True
@@ -261,19 +290,20 @@ class HostKeys (MutableMapping):
         hashed hostnames in the known_hosts file.
 
         :param str hostname: the hostname to hash
-        :param str salt: optional salt to use when hashing (must be 20 bytes long)
+        :param str salt: optional salt to use when hashing
+            (must be 20 bytes long)
         :return: the hashed hostname as a `str`
         """
         if salt is None:
             salt = os.urandom(sha1().digest_size)
         else:
-            if salt.startswith('|1|'):
-                salt = salt.split('|')[2]
+            if salt.startswith("|1|"):
+                salt = salt.split("|")[2]
             salt = decodebytes(b(salt))
         assert len(salt) == sha1().digest_size
         hmac = HMAC(salt, b(hostname), sha1).digest()
-        hostkey = '|1|%s|%s' % (u(encodebytes(salt)), u(encodebytes(hmac)))
-        return hostkey.replace('\n', '')
+        hostkey = "|1|%s|%s" % (u(encodebytes(salt)), u(encodebytes(hmac)))
+        return hostkey.replace("\n", "")
 
 
 class InvalidHostKey(Exception):
@@ -306,28 +336,32 @@ class HostKeyEntry:
 
         :param str line: a line from an OpenSSH known_hosts file
         """
-        log = get_logger('paramiko.hostkeys')
-        fields = line.split(' ')
+        log = get_logger("paramiko.hostkeys")
+        fields = line.split(" ")
         if len(fields) < 3:
             # Bad number of fields
-            log.info("Not enough fields found in known_hosts in line %s (%r)" %
-                     (lineno, line))
+            log.info(
+                "Not enough fields found in known_hosts in line %s (%r)"
+                % (lineno, line)
+            )
             return None
         fields = fields[:3]
 
         names, keytype, key = fields
-        names = names.split(',')
+        names = names.split(",")
 
         # Decide what kind of key we're looking at and create an object
         # to hold it accordingly.
         try:
             key = b(key)
-            if keytype == 'ssh-rsa':
+            if keytype == "ssh-rsa":
                 key = RSAKey(data=decodebytes(key))
-            elif keytype == 'ssh-dss':
+            elif keytype == "ssh-dss":
                 key = DSSKey(data=decodebytes(key))
-            elif keytype == 'ecdsa-sha2-nistp256':
+            elif keytype in ECDSAKey.supported_key_format_identifiers():
                 key = ECDSAKey(data=decodebytes(key), validate_point=False)
+            elif keytype == "ssh-ed25519":
+                key = Ed25519Key(data=decodebytes(key))
             else:
                 log.info("Unable to handle key of type %s" % (keytype,))
                 return None
@@ -344,9 +378,12 @@ class HostKeyEntry:
         included.
         """
         if self.valid:
-            return '%s %s %s\n' % (','.join(self.hostnames), self.key.get_name(),
-                   self.key.get_base64())
+            return "%s %s %s\n" % (
+                ",".join(self.hostnames),
+                self.key.get_name(),
+                self.key.get_base64(),
+            )
         return None
 
     def __repr__(self):
-        return '<HostKeyEntry %r: %r>' % (self.hostnames, self.key)
+        return "<HostKeyEntry %r: %r>" % (self.hostnames, self.key)
