@@ -22,7 +22,7 @@ from paramiko.common import x80000000, o700, o70, xffffffff
 from paramiko.py3compat import long, b
 
 
-class SFTPAttributes (object):
+class SFTPAttributes(object):
     """
     Representation of the attributes of a file (or proxied file) for SFTP in
     client or server mode.  It attemps to mirror the object returned by
@@ -82,9 +82,9 @@ class SFTPAttributes (object):
         return attr
 
     def __repr__(self):
-        return '<SFTPAttributes: %s>' % self._debug_str()
+        return "<SFTPAttributes: %s>" % self._debug_str()
 
-    ###  internals...
+    # ...internals...
     @classmethod
     def _from_msg(cls, msg, filename=None, longname=None):
         attr = cls()
@@ -144,29 +144,29 @@ class SFTPAttributes (object):
         return
 
     def _debug_str(self):
-        out = '[ '
+        out = "[ "
         if self.st_size is not None:
-            out += 'size=%d ' % self.st_size
+            out += "size=%d " % self.st_size
         if (self.st_uid is not None) and (self.st_gid is not None):
-            out += 'uid=%d gid=%d ' % (self.st_uid, self.st_gid)
+            out += "uid=%d gid=%d " % (self.st_uid, self.st_gid)
         if self.st_mode is not None:
-            out += 'mode=' + oct(self.st_mode) + ' '
+            out += "mode=" + oct(self.st_mode) + " "
         if (self.st_atime is not None) and (self.st_mtime is not None):
-            out += 'atime=%d mtime=%d ' % (self.st_atime, self.st_mtime)
+            out += "atime=%d mtime=%d " % (self.st_atime, self.st_mtime)
         for k, v in self.attr.items():
             out += '"%s"=%r ' % (str(k), v)
-        out += ']'
+        out += "]"
         return out
 
     @staticmethod
     def _rwx(n, suid, sticky=False):
         if suid:
             suid = 2
-        out = '-r'[n >> 2] + '-w'[(n >> 1) & 1]
+        out = "-r"[n >> 2] + "-w"[(n >> 1) & 1]
         if sticky:
-            out += '-xTt'[suid + (n & 1)]
+            out += "-xTt"[suid + (n & 1)]
         else:
-            out += '-xSs'[suid + (n & 1)]
+            out += "-xSs"[suid + (n & 1)]
         return out
 
     def __str__(self):
@@ -174,47 +174,67 @@ class SFTPAttributes (object):
         if self.st_mode is not None:
             kind = stat.S_IFMT(self.st_mode)
             if kind == stat.S_IFIFO:
-                ks = 'p'
+                ks = "p"
             elif kind == stat.S_IFCHR:
-                ks = 'c'
+                ks = "c"
             elif kind == stat.S_IFDIR:
-                ks = 'd'
+                ks = "d"
             elif kind == stat.S_IFBLK:
-                ks = 'b'
+                ks = "b"
             elif kind == stat.S_IFREG:
-                ks = '-'
+                ks = "-"
             elif kind == stat.S_IFLNK:
-                ks = 'l'
+                ks = "l"
             elif kind == stat.S_IFSOCK:
-                ks = 's'
+                ks = "s"
             else:
-                ks = '?'
-            ks += self._rwx((self.st_mode & o700) >> 6, self.st_mode & stat.S_ISUID)
-            ks += self._rwx((self.st_mode & o70) >> 3, self.st_mode & stat.S_ISGID)
-            ks += self._rwx(self.st_mode & 7, self.st_mode & stat.S_ISVTX, True)
+                ks = "?"
+            ks += self._rwx(
+                (self.st_mode & o700) >> 6, self.st_mode & stat.S_ISUID
+            )
+            ks += self._rwx(
+                (self.st_mode & o70) >> 3, self.st_mode & stat.S_ISGID
+            )
+            ks += self._rwx(
+                self.st_mode & 7, self.st_mode & stat.S_ISVTX, True
+            )
         else:
-            ks = '?---------'
+            ks = "?---------"
         # compute display date
         if (self.st_mtime is None) or (self.st_mtime == xffffffff):
             # shouldn't really happen
-            datestr = '(unknown date)'
+            datestr = "(unknown date)"
         else:
             if abs(time.time() - self.st_mtime) > 15552000:
                 # (15552000 = 6 months)
-                datestr = time.strftime('%d %b %Y', time.localtime(self.st_mtime))
+                datestr = time.strftime(
+                    "%d %b %Y", time.localtime(self.st_mtime)
+                )
             else:
-                datestr = time.strftime('%d %b %H:%M', time.localtime(self.st_mtime))
-        filename = getattr(self, 'filename', '?')
+                datestr = time.strftime(
+                    "%d %b %H:%M", time.localtime(self.st_mtime)
+                )
+        filename = getattr(self, "filename", "?")
 
         # not all servers support uid/gid
         uid = self.st_uid
         gid = self.st_gid
+        size = self.st_size
         if uid is None:
             uid = 0
         if gid is None:
             gid = 0
+        if size is None:
+            size = 0
 
-        return '%s   1 %-8d %-8d %8d %-12s %s' % (ks, uid, gid, self.st_size, datestr, filename)
+        return "%s   1 %-8d %-8d %8d %-12s %s" % (
+            ks,
+            uid,
+            gid,
+            size,
+            datestr,
+            filename,
+        )
 
     def asbytes(self):
         return b(str(self))
