@@ -29,12 +29,39 @@ from paramiko.channel import Channel
 from paramiko.message import Message
 from paramiko.common import INFO, DEBUG, o777
 from paramiko.py3compat import bytestring, b, u, long
-from paramiko.sftp import BaseSFTP, CMD_OPENDIR, CMD_HANDLE, SFTPError, CMD_READDIR, \
-    CMD_NAME, CMD_CLOSE, SFTP_FLAG_READ, SFTP_FLAG_WRITE, SFTP_FLAG_CREATE, \
-    SFTP_FLAG_TRUNC, SFTP_FLAG_APPEND, SFTP_FLAG_EXCL, CMD_OPEN, CMD_REMOVE, \
-    CMD_RENAME, CMD_MKDIR, CMD_RMDIR, CMD_STAT, CMD_ATTRS, CMD_LSTAT, \
-    CMD_SYMLINK, CMD_SETSTAT, CMD_READLINK, CMD_REALPATH, CMD_STATUS, SFTP_OK, \
-    SFTP_EOF, SFTP_NO_SUCH_FILE, SFTP_PERMISSION_DENIED
+from paramiko.sftp import (
+    BaseSFTP,
+    CMD_OPENDIR,
+    CMD_HANDLE,
+    SFTPError,
+    CMD_READDIR,
+    CMD_NAME,
+    CMD_CLOSE,
+    SFTP_FLAG_READ,
+    SFTP_FLAG_WRITE,
+    SFTP_FLAG_CREATE,
+    SFTP_FLAG_TRUNC,
+    SFTP_FLAG_APPEND,
+    SFTP_FLAG_EXCL,
+    CMD_OPEN,
+    CMD_REMOVE,
+    CMD_RENAME,
+    CMD_MKDIR,
+    CMD_RMDIR,
+    CMD_STAT,
+    CMD_ATTRS,
+    CMD_LSTAT,
+    CMD_SYMLINK,
+    CMD_SETSTAT,
+    CMD_READLINK,
+    CMD_REALPATH,
+    CMD_STATUS,
+    CMD_EXTENDED,
+    SFTP_OK,
+    SFTP_EOF,
+    SFTP_NO_SUCH_FILE,
+    SFTP_PERMISSION_DENIED,
+)
 
 from paramiko.sftp_attr import SFTPAttributes
 from paramiko.ssh_exception import SSHException
@@ -49,14 +76,15 @@ def _to_unicode(s):
     probably doesn't know the filename's encoding.
     """
     try:
-        return s.encode('ascii')
+        return s.encode("ascii")
     except (UnicodeError, AttributeError):
         try:
-            return s.decode('utf-8')
+            return s.decode("utf-8")
         except UnicodeError:
             return s
 
-b_slash = b'/'
+
+b_slash = b"/"
 
 
 class SFTPClient(BaseSFTP, ClosingContextManager):
@@ -68,6 +96,7 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
 
     Instances of this class may be used as context managers.
     """
+
     def __init__(self, sock):
         """
         Create an SFTP client from an existing `.Channel`.  The channel
@@ -78,8 +107,8 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
 
         :param .Channel sock: an open `.Channel` using the ``"sftp"`` subsystem
 
-        :raises SSHException: if there's an exception while negotiating
-            sftp
+        :raises:
+            `.SSHException` -- if there's an exception while negotiating sftp
         """
         BaseSFTP.__init__(self)
         self.sock = sock
@@ -93,13 +122,17 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         if type(sock) is Channel:
             # override default logger
             transport = self.sock.get_transport()
-            self.logger = util.get_logger(transport.get_log_channel() + '.sftp')
+            self.logger = util.get_logger(
+                transport.get_log_channel() + ".sftp"
+            )
             self.ultra_debug = transport.get_hexdump()
         try:
             server_version = self._send_version()
         except EOFError:
-            raise SSHException('EOF during negotiation')
-        self._log(INFO, 'Opened sftp connection (server version %d)' % server_version)
+            raise SSHException("EOF during negotiation")
+        self._log(
+            INFO, "Opened sftp connection (server version %d)" % server_version
+        )
 
     @classmethod
     def from_transport(cls, t, window_size=None, max_packet_size=None):
@@ -111,7 +144,8 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         OpenSSH and should work adequately for both files transfers and
         interactive sessions.
 
-        :param .Transport t: an open `.Transport` which is already authenticated
+        :param .Transport t: an open `.Transport` which is already
+            authenticated
         :param int window_size:
             optional window size for the `.SFTPClient` session.
         :param int max_packet_size:
@@ -124,11 +158,12 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         .. versionchanged:: 1.15
             Added the ``window_size`` and ``max_packet_size`` arguments.
         """
-        chan = t.open_session(window_size=window_size,
-                              max_packet_size=max_packet_size)
+        chan = t.open_session(
+            window_size=window_size, max_packet_size=max_packet_size
+        )
         if chan is None:
             return None
-        chan.invoke_subsystem('sftp')
+        chan.invoke_subsystem("sftp")
         return cls(chan)
 
     def _log(self, level, msg, *args):
@@ -136,9 +171,14 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
             for m in msg:
                 self._log(level, m, *args)
         else:
-            # escape '%' in msg (they could come from file or directory names) before logging
-            msg = msg.replace('%','%%')
-            super(SFTPClient, self)._log(level, "[chan %s] " + msg, *([self.sock.get_name()] + list(args)))
+            # escape '%' in msg (they could come from file or directory names)
+            # before logging
+            msg = msg.replace("%", "%%")
+            super(SFTPClient, self)._log(
+                level,
+                "[chan %s] " + msg,
+                *([self.sock.get_name()] + list(args))
+            )
 
     def close(self):
         """
@@ -146,7 +186,7 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
 
         .. versionadded:: 1.4
         """
-        self._log(INFO, 'sftp session closed.')
+        self._log(INFO, "sftp session closed.")
         self.sock.close()
 
     def get_channel(self):
@@ -158,9 +198,10 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         """
         return self.sock
 
-    def listdir(self, path='.'):
+    def listdir(self, path="."):
         """
-        Return a list containing the names of the entries in the given ``path``.
+        Return a list containing the names of the entries in the given
+        ``path``.
 
         The list is in arbitrary order.  It does not include the special
         entries ``'.'`` and ``'..'`` even if they are present in the folder.
@@ -171,7 +212,7 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         """
         return [f.filename for f in self.listdir_attr(path)]
 
-    def listdir_attr(self, path='.'):
+    def listdir_attr(self, path="."):
         """
         Return a list containing `.SFTPAttributes` objects corresponding to
         files in the given ``path``.  The list is in arbitrary order.  It does
@@ -189,10 +230,10 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         .. versionadded:: 1.2
         """
         path = self._adjust_cwd(path)
-        self._log(DEBUG, 'listdir(%r)' % path)
+        self._log(DEBUG, "listdir(%r)" % path)
         t, msg = self._request(CMD_OPENDIR, path)
         if t != CMD_HANDLE:
-            raise SFTPError('Expected handle')
+            raise SFTPError("Expected handle")
         handle = msg.get_binary()
         filelist = []
         while True:
@@ -202,18 +243,18 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
                 # done with handle
                 break
             if t != CMD_NAME:
-                raise SFTPError('Expected name response')
+                raise SFTPError("Expected name response")
             count = msg.get_int()
             for i in range(count):
                 filename = msg.get_text()
                 longname = msg.get_text()
                 attr = SFTPAttributes._from_msg(msg, filename, longname)
-                if (filename != '.') and (filename != '..'):
+                if (filename != ".") and (filename != ".."):
                     filelist.append(attr)
         self._request(CMD_CLOSE, handle)
         return filelist
 
-    def listdir_iter(self, path='.', read_aheads=50):
+    def listdir_iter(self, path=".", read_aheads=50):
         """
         Generator version of `.listdir_attr`.
 
@@ -228,11 +269,11 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         .. versionadded:: 1.15
         """
         path = self._adjust_cwd(path)
-        self._log(DEBUG, 'listdir(%r)' % path)
+        self._log(DEBUG, "listdir(%r)" % path)
         t, msg = self._request(CMD_OPENDIR, path)
 
         if t != CMD_HANDLE:
-            raise SFTPError('Expected handle')
+            raise SFTPError("Expected handle")
 
         handle = msg.get_string()
 
@@ -247,7 +288,6 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
                     num = self._async_request(type(None), CMD_READDIR, handle)
                     nums.append(num)
 
-
                 # For each of our sent requests
                 # Read and parse the corresponding packets
                 # If we're at the end of our queued requests, then fire off
@@ -266,8 +306,9 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
                         filename = msg.get_text()
                         longname = msg.get_text()
                         attr = SFTPAttributes._from_msg(
-                            msg, filename, longname)
-                        if (filename != '.') and (filename != '..'):
+                            msg, filename, longname
+                        )
+                        if (filename != ".") and (filename != ".."):
                             yield attr
 
                 # If we've hit the end of our queued requests, reset nums.
@@ -277,8 +318,7 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
                 self._request(CMD_CLOSE, handle)
                 return
 
-
-    def open(self, filename, mode='r', bufsize=-1):
+    def open(self, filename, mode="r", bufsize=-1):
         """
         Open a file on the remote server.  The arguments are the same as for
         Python's built-in `python:file` (aka `python:open`).  A file-like
@@ -308,27 +348,29 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         :param int bufsize: desired buffering (-1 = default buffer size)
         :return: an `.SFTPFile` object representing the open file
 
-        :raises IOError: if the file could not be opened.
+        :raises: ``IOError`` -- if the file could not be opened.
         """
         filename = self._adjust_cwd(filename)
-        self._log(DEBUG, 'open(%r, %r)' % (filename, mode))
+        self._log(DEBUG, "open(%r, %r)" % (filename, mode))
         imode = 0
-        if ('r' in mode) or ('+' in mode):
+        if ("r" in mode) or ("+" in mode):
             imode |= SFTP_FLAG_READ
-        if ('w' in mode) or ('+' in mode) or ('a' in mode):
+        if ("w" in mode) or ("+" in mode) or ("a" in mode):
             imode |= SFTP_FLAG_WRITE
-        if 'w' in mode:
+        if "w" in mode:
             imode |= SFTP_FLAG_CREATE | SFTP_FLAG_TRUNC
-        if 'a' in mode:
+        if "a" in mode:
             imode |= SFTP_FLAG_CREATE | SFTP_FLAG_APPEND
-        if 'x' in mode:
+        if "x" in mode:
             imode |= SFTP_FLAG_CREATE | SFTP_FLAG_EXCL
         attrblock = SFTPAttributes()
         t, msg = self._request(CMD_OPEN, filename, imode, attrblock)
         if t != CMD_HANDLE:
-            raise SFTPError('Expected handle')
+            raise SFTPError("Expected handle")
         handle = msg.get_binary()
-        self._log(DEBUG, 'open(%r, %r) -> %s' % (filename, mode, hexlify(handle)))
+        self._log(
+            DEBUG, "open(%r, %r) -> %s" % (filename, mode, u(hexlify(handle)))
+        )
         return SFTPFile(self, handle, mode, bufsize)
 
     # Python continues to vacillate about "open" vs "file"...
@@ -341,10 +383,10 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
 
         :param str path: path (absolute or relative) of the file to remove
 
-        :raises IOError: if the path refers to a folder (directory)
+        :raises: ``IOError`` -- if the path refers to a folder (directory)
         """
         path = self._adjust_cwd(path)
-        self._log(DEBUG, 'remove(%r)' % path)
+        self._log(DEBUG, "remove(%r)" % path)
         self._request(CMD_REMOVE, path)
 
     unlink = remove
@@ -353,17 +395,47 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         """
         Rename a file or folder from ``oldpath`` to ``newpath``.
 
-        :param str oldpath: existing name of the file or folder
-        :param str newpath: new name for the file or folder
+        .. note::
+            This method implements 'standard' SFTP ``RENAME`` behavior; those
+            seeking the OpenSSH "POSIX rename" extension behavior should use
+            `posix_rename`.
 
-        :raises IOError: if ``newpath`` is a folder, or something else goes
+        :param str oldpath:
+            existing name of the file or folder
+        :param str newpath:
+            new name for the file or folder, must not exist already
+
+        :raises:
+            ``IOError`` -- if ``newpath`` is a folder, or something else goes
             wrong
         """
         oldpath = self._adjust_cwd(oldpath)
         newpath = self._adjust_cwd(newpath)
-        self._log(DEBUG, 'rename(%r, %r)' % (oldpath, newpath))
+        self._log(DEBUG, "rename(%r, %r)" % (oldpath, newpath))
         self._request(CMD_RENAME, oldpath, newpath)
 
+    def posix_rename(self, oldpath, newpath):
+        """
+        Rename a file or folder from ``oldpath`` to ``newpath``, following
+        posix conventions.
+
+        :param str oldpath: existing name of the file or folder
+        :param str newpath: new name for the file or folder, will be
+            overwritten if it already exists
+
+        :raises:
+            ``IOError`` -- if ``newpath`` is a folder, posix-rename is not
+            supported by the server or something else goes wrong
+
+        :versionadded: 2.2
+        """
+        oldpath = self._adjust_cwd(oldpath)
+        newpath = self._adjust_cwd(newpath)
+        self._log(DEBUG, "posix_rename(%r, %r)" % (oldpath, newpath))
+        self._request(
+            CMD_EXTENDED, "posix-rename@openssh.com", oldpath, newpath
+        )
+
     def mkdir(self, path, mode=o777):
         """
         Create a folder (directory) named ``path`` with numeric mode ``mode``.
@@ -374,7 +446,7 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         :param int mode: permissions (posix-style) for the newly-created folder
         """
         path = self._adjust_cwd(path)
-        self._log(DEBUG, 'mkdir(%r, %r)' % (path, mode))
+        self._log(DEBUG, "mkdir(%r, %r)" % (path, mode))
         attr = SFTPAttributes()
         attr.st_mode = mode
         self._request(CMD_MKDIR, path, attr)
@@ -386,7 +458,7 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         :param str path: name of the folder to remove
         """
         path = self._adjust_cwd(path)
-        self._log(DEBUG, 'rmdir(%r)' % path)
+        self._log(DEBUG, "rmdir(%r)" % path)
         self._request(CMD_RMDIR, path)
 
     def stat(self, path):
@@ -409,10 +481,10 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
             file
         """
         path = self._adjust_cwd(path)
-        self._log(DEBUG, 'stat(%r)' % path)
+        self._log(DEBUG, "stat(%r)" % path)
         t, msg = self._request(CMD_STAT, path)
         if t != CMD_ATTRS:
-            raise SFTPError('Expected attributes')
+            raise SFTPError("Expected attributes")
         return SFTPAttributes._from_msg(msg)
 
     def lstat(self, path):
@@ -427,22 +499,21 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
             file
         """
         path = self._adjust_cwd(path)
-        self._log(DEBUG, 'lstat(%r)' % path)
+        self._log(DEBUG, "lstat(%r)" % path)
         t, msg = self._request(CMD_LSTAT, path)
         if t != CMD_ATTRS:
-            raise SFTPError('Expected attributes')
+            raise SFTPError("Expected attributes")
         return SFTPAttributes._from_msg(msg)
 
     def symlink(self, source, dest):
         """
-        Create a symbolic link (shortcut) of the ``source`` path at
-        ``destination``.
+        Create a symbolic link to the ``source`` path at ``destination``.
 
         :param str source: path of the original file
         :param str dest: path of the newly created symlink
         """
         dest = self._adjust_cwd(dest)
-        self._log(DEBUG, 'symlink(%r, %r)' % (source, dest))
+        self._log(DEBUG, "symlink(%r, %r)" % (source, dest))
         source = bytestring(source)
         self._request(CMD_SYMLINK, source, dest)
 
@@ -456,7 +527,7 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         :param int mode: new permissions
         """
         path = self._adjust_cwd(path)
-        self._log(DEBUG, 'chmod(%r, %r)' % (path, mode))
+        self._log(DEBUG, "chmod(%r, %r)" % (path, mode))
         attr = SFTPAttributes()
         attr.st_mode = mode
         self._request(CMD_SETSTAT, path, attr)
@@ -473,19 +544,19 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         :param int gid: new group id
         """
         path = self._adjust_cwd(path)
-        self._log(DEBUG, 'chown(%r, %r, %r)' % (path, uid, gid))
+        self._log(DEBUG, "chown(%r, %r, %r)" % (path, uid, gid))
         attr = SFTPAttributes()
         attr.st_uid, attr.st_gid = uid, gid
         self._request(CMD_SETSTAT, path, attr)
 
     def utime(self, path, times):
         """
-        Set the access and modified times of the file specified by ``path``.  If
-        ``times`` is ``None``, then the file's access and modified times are set
-        to the current time.  Otherwise, ``times`` must be a 2-tuple of numbers,
-        of the form ``(atime, mtime)``, which is used to set the access and
-        modified times, respectively.  This bizarre API is mimicked from Python
-        for the sake of consistency -- I apologize.
+        Set the access and modified times of the file specified by ``path``.
+        If ``times`` is ``None``, then the file's access and modified times
+        are set to the current time.  Otherwise, ``times`` must be a 2-tuple
+        of numbers, of the form ``(atime, mtime)``, which is used to set the
+        access and modified times, respectively.  This bizarre API is mimicked
+        from Python for the sake of consistency -- I apologize.
 
         :param str path: path of the file to modify
         :param tuple times:
@@ -495,7 +566,7 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         path = self._adjust_cwd(path)
         if times is None:
             times = (time.time(), time.time())
-        self._log(DEBUG, 'utime(%r, %r)' % (path, times))
+        self._log(DEBUG, "utime(%r, %r)" % (path, times))
         attr = SFTPAttributes()
         attr.st_atime, attr.st_mtime = times
         self._request(CMD_SETSTAT, path, attr)
@@ -507,11 +578,10 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         method on Python file objects.
 
         :param str path: path of the file to modify
-        :param size: the new size of the file
-        :type size: int or long
+        :param int size: the new size of the file
         """
         path = self._adjust_cwd(path)
-        self._log(DEBUG, 'truncate(%r, %r)' % (path, size))
+        self._log(DEBUG, "truncate(%r, %r)" % (path, size))
         attr = SFTPAttributes()
         attr.st_size = size
         self._request(CMD_SETSTAT, path, attr)
@@ -526,15 +596,15 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         :return: target path, as a `str`
         """
         path = self._adjust_cwd(path)
-        self._log(DEBUG, 'readlink(%r)' % path)
+        self._log(DEBUG, "readlink(%r)" % path)
         t, msg = self._request(CMD_READLINK, path)
         if t != CMD_NAME:
-            raise SFTPError('Expected name response')
+            raise SFTPError("Expected name response")
         count = msg.get_int()
         if count == 0:
             return None
         if count != 1:
-            raise SFTPError('Readlink returned %d results' % count)
+            raise SFTPError("Readlink returned %d results" % count)
         return _to_unicode(msg.get_string())
 
     def normalize(self, path):
@@ -547,16 +617,16 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         :param str path: path to be normalized
         :return: normalized form of the given path (as a `str`)
 
-        :raises IOError: if the path can't be resolved on the server
+        :raises: ``IOError`` -- if the path can't be resolved on the server
         """
         path = self._adjust_cwd(path)
-        self._log(DEBUG, 'normalize(%r)' % path)
+        self._log(DEBUG, "normalize(%r)" % path)
         t, msg = self._request(CMD_REALPATH, path)
         if t != CMD_NAME:
-            raise SFTPError('Expected name response')
+            raise SFTPError("Expected name response")
         count = msg.get_int()
         if count != 1:
-            raise SFTPError('Realpath returned %d results' % count)
+            raise SFTPError("Realpath returned %d results" % count)
         return msg.get_text()
 
     def chdir(self, path=None):
@@ -570,7 +640,8 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
 
         :param str path: new current working directory
 
-        :raises IOError: if the requested path doesn't exist on the server
+        :raises:
+            ``IOError`` -- if the requested path doesn't exist on the server
 
         .. versionadded:: 1.4
         """
@@ -578,7 +649,9 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
             self._cwd = None
             return
         if not stat.S_ISDIR(self.stat(path).st_mode):
-            raise SFTPError(errno.ENOTDIR, "%s: %s" % (os.strerror(errno.ENOTDIR), path))
+            raise SFTPError(
+                errno.ENOTDIR, "%s: %s" % (os.strerror(errno.ENOTDIR), path)
+            )
         self._cwd = b(self.normalize(path))
 
     def getcwd(self):
@@ -631,7 +704,7 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
 
         .. versionadded:: 1.10
         """
-        with self.file(remotepath, 'wb') as fr:
+        with self.file(remotepath, "wb") as fr:
             fr.set_pipelined(True)
             size = self._transfer_with_callback(
                 reader=fl, writer=fr, file_size=file_size, callback=callback
@@ -639,7 +712,9 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         if confirm:
             s = self.stat(remotepath)
             if s.st_size != size:
-                raise IOError('size mismatch in put!  %d != %d' % (s.st_size, size))
+                raise IOError(
+                    "size mismatch in put!  %d != %d" % (s.st_size, size)
+                )
         else:
             s = SFTPAttributes()
         return s
@@ -663,7 +738,8 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
             whether to do a stat() on the file afterwards to confirm the file
             size
 
-        :return: an `.SFTPAttributes` object containing attributes about the given file
+        :return: an `.SFTPAttributes` object containing attributes about the
+            given file
 
         .. versionadded:: 1.4
         .. versionchanged:: 1.7.4
@@ -672,7 +748,7 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
             ``confirm`` param added.
         """
         file_size = os.stat(localpath).st_size
-        with open(localpath, 'rb') as fl:
+        with open(localpath, "rb") as fl:
             return self.putfo(fl, remotepath, file_size, callback, confirm)
 
     def getfo(self, remotepath, fl, callback=None):
@@ -693,7 +769,7 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         .. versionadded:: 1.10
         """
         file_size = self.stat(remotepath).st_size
-        with self.open(remotepath, 'rb') as fr:
+        with self.open(remotepath, "rb") as fr:
             fr.prefetch(file_size)
             return self._transfer_with_callback(
                 reader=fr, writer=fl, file_size=file_size, callback=callback
@@ -715,13 +791,15 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
         .. versionchanged:: 1.7.4
             Added the ``callback`` param
         """
-        with open(localpath, 'wb') as fl:
+        with open(localpath, "wb") as fl:
             size = self.getfo(remotepath, fl, callback)
         s = os.stat(localpath)
         if s.st_size != size:
-            raise IOError('size mismatch in get!  %d != %d' % (s.st_size, size))
+            raise IOError(
+                "size mismatch in get!  %d != %d" % (s.st_size, size)
+            )
 
-    ###  internals...
+    # ...internals...
 
     def _request(self, t, *arg):
         num = self._async_request(type(None), t, *arg)
@@ -757,14 +835,15 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
             try:
                 t, data = self._read_packet()
             except EOFError as e:
-                raise SSHException('Server connection dropped: %s' % str(e))
+                raise SSHException("Server connection dropped: %s" % str(e))
             msg = Message(data)
             num = msg.get_int()
             self._lock.acquire()
             try:
                 if num not in self._expecting:
-                    # might be response for a file that was closed before responses came back
-                    self._log(DEBUG, 'Unexpected response #%d' % (num,))
+                    # might be response for a file that was closed before
+                    # responses came back
+                    self._log(DEBUG, "Unexpected response #%d" % (num,))
                     if waitfor is None:
                         # just doing a single check
                         break
@@ -778,7 +857,10 @@ class SFTPClient(BaseSFTP, ClosingContextManager):
                 if t == CMD_STATUS:
                     self._convert_status(msg)
                 return t, msg
-            if fileobj is not type(None):
+
+            # can not rewrite this to deal with E721, either as a None check
+            # nor as not an instance of None or NoneType
+            if fileobj is not type(None):  # noqa
                 fileobj._async_response(t, msg, num)
             if waitfor is None:
                 # just doing a single check
@@ -828,4 +910,5 @@ class SFTP(SFTPClient):
     """
     An alias for `.SFTPClient` for backwards compatibility.
     """
+
     pass
